@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 
 #include "curl_setup.h"
@@ -27,8 +29,6 @@
 #include "urldata.h"
 #include "strcase.h"
 #include "strdup.h"
-#include "vauth/vauth.h"
-#include "vauth/digest.h"
 #include "http_aws_sigv4.h"
 #include "curl_sha256.h"
 #include "transfer.h"
@@ -202,12 +202,12 @@ CURLcode Curl_output_aws_sigv4(struct Curl_easy *data, bool proxy)
   if(!service) {
     tmp0 = hostname;
     tmp1 = strchr(tmp0, '.');
-    len = tmp1 - tmp0;
-    if(!tmp1 || len < 1) {
+    if(!tmp1) {
       infof(data, "service missing in parameters or hostname");
       ret = CURLE_URL_MALFORMAT;
       goto fail;
     }
+    len = tmp1 - tmp0;
     service = Curl_memdup(tmp0, len + 1);
     if(!service) {
       goto fail;
@@ -217,12 +217,12 @@ CURLcode Curl_output_aws_sigv4(struct Curl_easy *data, bool proxy)
     if(!region) {
       tmp0 = tmp1 + 1;
       tmp1 = strchr(tmp0, '.');
-      len = tmp1 - tmp0;
-      if(!tmp1 || len < 1) {
+      if(!tmp1) {
         infof(data, "region missing in parameters or hostname");
         ret = CURLE_URL_MALFORMAT;
         goto fail;
       }
+      len = tmp1 - tmp0;
       region = Curl_memdup(tmp0, len + 1);
       if(!region) {
         goto fail;
